module tinkermass
  use szclus_utils
  
  implicit none
  real*8 :: a1,a2,a3,a4

contains
  subroutine get_tinker_params(delta,redshift)
    real*8, intent(in) :: delta, redshift
    integer, parameter :: n = 9
    real*8, dimension(0:n-1) :: x,y,z !using IDL style indexing
    real*8 ztemp
    real*8 :: at
    integer i,fail
    if (abs(delta-200d0) .lt. 0.1) then 
       a1=0.18586589d0
       a2=1.4669040d0
       a3=2.5711040d0
       a4=1.1939580d0
    else
       ! initialize the overdensities
       do i=0,n-1,2
          x(i) = log10(200 * 2.0**(0.5*i))
       end do
       do i=0,n-2,2
          x(i+1) = log10(300 * 2.0**(0.5*i))
       end do

       ! first parameter
       if (delta .ge. 1600) then 
          a1 = 0.26
       else
          y(0) = 1.858659d-01
          y(1) = 1.995973d-01
          y(2) = 2.115659d-01
          y(3) = 2.184113d-01
          y(4) = 2.480968d-01
          y(5) = 2.546053d-01
          y(6) = 2.600000d-01
          y(7) = 2.600000d-01
          y(8) = 2.600000d-01
          call dspline(x,y,n,1d30,1d30,z)
          call dsplint(x,y,z,n,log10(delta),a1,fail)
!          z = SPL_INIT(X,Y,YP0=1.0e30,YPN_1=1.0e30)
 !         a1 = SPL_INTERP(X,Y,z,log10(delta))
       endif

       ! second parameter
       y(0) = 1.466904d+00
       y(1) = 1.521782d+00
       y(2) = 1.559186d+00
       y(3) = 1.614585d+00
       y(4) = 1.869936d+00
       y(5) = 2.128056d+00
       y(6) = 2.301275d+00
       y(7) = 2.529241d+00
       y(8) = 2.661983d+00
!       z = SPL_INIT(X,Y,YP0=1.0e30,YPN_1=1.0e30)
 !      a2 = SPL_INTERP(X,Y,z,log10(delta))
       call dspline(x,y,n,1d30,1d30,z)
       call dsplint(x,y,z,n,log10(delta),a2,fail)

       ! third parameter
       y(0) = 2.571104d+00
       y(1) = 2.254217d+00
       y(2) = 2.048674d+00
       y(3) = 1.869559d+00
       y(4) = 1.588649d+00
       y(5) = 1.507134d+00
       y(6) = 1.464374d+00
       y(7) = 1.436827d+00
       y(8) = 1.405210d+00
!       z = SPL_INIT(X,Y,YP0=1.0e30,YPN_1=1.0e30)
!       a3 = SPL_INTERP(X,Y,z,log10(delta))
       call dspline(x,y,n,1d30,1d30,z)
       call dsplint(x,y,z,n,log10(delta),a3,fail)
       ! fourth parameter
       y(0) = 1.193958d+00
       y(1) = 1.270316d+00
       y(2) = 1.335191d+00
       y(3) = 1.446266d+00
       y(4) = 1.581345d+00
       y(5) = 1.795050d+00
       y(6) = 1.965613d+00
       y(7) = 2.237466d+00
       y(8) = 2.439729d+00
!       z = SPL_INIT(X,Y,YP0=1.0e30,YPN_1=1.0e30)
!       a4 = SPL_INTERP(X,Y,z,log10(delta))
       call dspline(x,y,n,1d30,1d30,z)
       call dsplint(x,y,z,n,log10(delta),a4,fail)
   endif
    ! now adjust for redshift
    if (redshift .lt. 0d0) then        
       return
    endif
       
    ztemp = redshift
    if(redshift .gt. 3d0) then 
       ztemp = 3d0
    endif
    a1 = a1*(1.0+ztemp)**(-0.14)
    a2 = a2*(1.0+ztemp)**(-0.06)
    if (delta .gt. 75.0) then 
       at = -(0.75/log10(delta/75.0))**(1.2)
       at = 10d0**(at)
    else
       at = 1d0
    endif
    a3 = a3*(1.0+ztemp)**(-1.0*at)
  end subroutine get_tinker_params


  subroutine tinker_f_sigma(sigma, nSigma,z,delta, fsig)
    real*8, intent(in) :: z,delta
    integer, intent(in) :: nSigma
    real*8, dimension(:), intent(in) :: sigma
    real*8, dimension(:), intent(out) ::  fsig
    integer i
!get f(sigma) for Tinker mass func.
    call get_tinker_params(delta,z)


    do i=1,nSigma 
       if (a4/sigma(i)**2 .gt. 708.) then
          fsig(i) = 0d0
       else
          fsig(i) = a1*((sigma(i)/a3)**(-1d0*a2)+1d0)*exp(-1d0*a4/sigma(i)**2)
       endif
    enddo
  end subroutine tinker_f_sigma


end module tinkermass
